#pragma once
#ifndef PARSODUS_BACKENDMANAGER_H
#define PARSODUS_BACKENDMANAGER_H

#include <memory>
#include <string>
#include <vector>

#include "Parsodus/backend.h"

namespace pds {
    /**
     * A manager for backends
     * Aggregates and allow to search for backends that can process a specific language
     */
    class BackendManager {
        public:
            /**
             * Add a backend to the list of registered backends
             *
             * @param backend The backend to register
             */
            void registerBackend(std::unique_ptr<Backend> backend);

            /**
             * Register an *LR type backend
             */
            void registerLRBackend(std::unique_ptr<LRBackend> b);

            /**
             * Get a backend that can process the given language
             * The manager retains ownership of the returned pointer
             *
             * @param lang The language the backend should be able to process
             * @returns A pointer to a Backend if it can find one, nullptr otherwise
             */
            Backend* findBackend(std::string lang, std::string parserType);

        private:
            std::vector<std::unique_ptr<Backend> > m_backends; ///< The list of registered backends
    };
}

#endif //PARSODUS_BACKENDMANAGER_H

