#pragma once
#ifndef PARSODUS_GRAMMAR_H

#include <map>
#include <set>
#include <string>
#include <vector>

namespace pds {
    /**
     * Represents a grammar rule
     * head -> tail[0] tail[1] ... tail[tail.size() - 1]
     */
    struct Rule {
        std::string head; ///< The replaced variable
        std::vector<std::string> tail; ///< The replacement rule

        bool operator<(const Rule& other) const {
            if(head != other.head){
                return head < other.head;
            } else {
                return tail < other.tail;
            }
        }
    };

    /**
     * A context free grammar
     * Keeps track of variables, terminals and replacement rules
     */
    struct Grammar {
        std::string start; ///< the starting variable
        std::set<std::string> variables; ///< the variables
        std::set<std::string> terminals; ///< the terminals
        std::map<std::string, std::set<Rule> > rules; ///< the replacement rules
    };
}

#endif //PARSODUS_GRAMMAR_H
