#pragma once
#ifndef PARSODUS_LRTABLES_GENERATOR_H_YW3GIUNH
#define PARSODUS_LRTABLES_GENERATOR_H_YW3GIUNH

#include "Parsodus/grammar.h"
#include "Parsodus/lrtables/table.h"

namespace pds {
namespace lr {

/**
 * Base class for LR (and derivative) table generators (such as SLR and LALR)
 * Parametrized on the type of item to be used in the configuration sets
 */
template <typename Item>
class Generator {
    public:
        /**
         * Constructor
         *
         * @param start The start symbol for the grammar
         * @param g The grammar to translate
         */
        Generator(const std::string& start, const Grammar& g);

        /**
         * Generate an LRTable based on given grammar
         *
         * @returns An LR (or derivative) table for the grammar given at construction
         */
        LRTable generate();

    protected:
        /**
         * Indicate whether this specific algorithm needs to have the First and Follow sets generated
         */
        virtual bool needsFollowSet() = 0;

        /**
         * Build the starting item to build all item sets from
         *
         * @param startrule The constructed extended starting rule
         * @param eof The token used as end of file
         */
        virtual Item initial_item(Rule startrule, std::string eof) = 0;


        std::set<std::string> first(std::string s);
        std::set<std::string> follow(std::string s);

    private:
        /**
         * Build the `First` set
         */
        void buildFirst();

        /**
         * Build the `Follow` set
         */
        void buildFollow();

        /**
         * Compute the closure of an item set
         */
        std::set<Item> closure(const std::set<Item>& its);

        Grammar m_gram;
        Rule m_startrule;
        std::map<std::string, std::set<std::string>> m_first;
        std::map<std::string, std::set<std::string>> m_follow;
};

} /* lr */
} /* pdf */

#endif /* PARSODUS_LRTABLES_GENERATOR_H_YW3GIUNH */
